@@ -820,6 +820,10 @@ def interpret_items(schemas, raw_items, db_model=None):
                         'status': FetchStatusSchema.get_unknown_status(),
                         'header': header,
                         'rows': rows,
+                        'sort_keys': [
+                            processed_bundle_info["sort_key"]
+                            for processed_bundle_info in processed_bundle_infos
+                        ],
                     }
                 )
                 .data
@@ -878,7 +882,14 @@ def interpret_items(schemas, raw_items, db_model=None):
             return
 
         blocks.append(
-            SubworksheetsBlock().load({'subworksheet_infos': copy.deepcopy(worksheet_infos)}).data
+            SubworksheetsBlock()
+            .load(
+                {
+                    'subworksheet_infos': copy.deepcopy(worksheet_infos),
+                    'sort_keys': [worksheet_info["sort_key"] for worksheet_info in worksheet_infos],
+                }
+            )
+            .data
         )
 
         worksheet_infos[:] = []
@@ -910,9 +921,11 @@ def interpret_items(schemas, raw_items, db_model=None):
                 current_schema = None
 
             if item_type == TYPE_BUNDLE:
+                bundle_info = dict(bundle_info, sort_key=sort_key)
                 raw_to_block.append((len(blocks), len(bundle_infos)))
                 bundle_infos.append((raw_index, bundle_info))
             elif item_type == TYPE_WORKSHEET:
+                subworksheet_info = dict(subworksheet_info, sort_key=sort_key)
                 raw_to_block.append((len(blocks), len(worksheet_infos)))
                 worksheet_infos.append(subworksheet_info)
             elif item_type == TYPE_MARKUP:
@@ -981,7 +994,12 @@ def interpret_items(schemas, raw_items, db_model=None):
                     # Show item placeholders in brief mode
                     blocks.append(
                         PlaceholderBlockSchema()
-                        .load({'directive': formatting.tokens_to_string(value_obj)})
+                        .load(
+                            {
+                                'directive': formatting.tokens_to_string(value_obj),
+                                'sort_keys': [sort_key],
+                            }
+                        )
                         .data
                     )
 
