@@ -57,6 +57,7 @@ class Worker:
         work_dir,  # type: str
         local_bundles_dir,  # type: Optional[str]
         exit_when_idle,  # type: str
+        exit_after_num_runs,  # type: int
         idle_seconds,  # type: int
         bundle_service,  # type: BundleServiceClient
         shared_file_system,  # type: bool
@@ -67,6 +68,8 @@ class Worker:
         pass_down_termination=False,  # type: bool
         # A flag indicating if the work_dir will be deleted when the worker exits.
         delete_work_dir_on_exit=False,  # type: bool
+        # A flag indicating if the worker will exit if it encounters an exception
+        exit_on_exception=False,  # type: bool
     ):
         self.image_manager = image_manager
         self.dependency_manager = dependency_manager
@@ -93,11 +96,14 @@ class Worker:
         self.delete_work_dir_on_exit = delete_work_dir_on_exit
 
         self.exit_when_idle = exit_when_idle
+        self.exit_after_num_runs = exit_after_num_runs
+        self.num_runs = 0
         self.idle_seconds = idle_seconds
 
         self.terminate = False
         self.terminate_and_restage = False
         self.pass_down_termination = pass_down_termination
+        self.exit_on_exception = exit_on_exception
 
         self.last_checkin_successful = False
         self.last_time_ran = None  # type: Optional[bool]
@@ -177,6 +183,14 @@ class Worker:
         is_idle = now - self.last_time_ran > self.idle_seconds
         return self.exit_when_idle and is_idle and self.last_checkin_successful
 
+    def check_num_runs_stop(self):
+        """
+        Checks whether the worker has finished the number of job allowed to run.
+        :return: True if the number of jobs allowed to run is 0 and all those runs are finished.
+                 False if neither of the two conditions are met.
+        """
+        return self.exit_after_num_runs == self.num_runs and len(self.runs) == 0
+
     def start(self):
         """Return whether we ran anything."""
         self.load_state()
@@ -190,14 +204,18 @@ class Worker:
                 self.checkin()
                 self.check_termination()
                 self.save_state()
-                if self.check_idle_stop():
+                if self.check_idle_stop() or self.check_num_runs_stop():
                     self.terminate = True
             except Exception:
                 self.last_checkin_successful = False
                 traceback.print_exc()
-                # Sleep for a long time so we don't keep on failing.
-                logger.error('Sleeping for 1 hour due to exception...please help me!')
-                time.sleep(1 * 60 * 60)
+                if self.exit_on_exception:
+                    logger.error('Encountered exception, terminating the worker...')
+                    self.terminate = True
+                else:
+                    # Sleep for a long time so we don't keep on failing.
+                    logger.error('Sleeping for 1 hour due to exception...please help me!')
+                    time.sleep(1 * 60 * 60)
         self.cleanup()
 
     def cleanup(self):
@@ -303,6 +321,7 @@ class Worker:
             'runs': [run.as_dict for run in self.all_runs],
             'shared_file_system': self.shared_file_system,
             'tag_exclusive': self.tag_exclusive,
+            'exit_after_num_runs': self.exit_after_num_runs - self.num_runs,
         }
         try:
             response = self.bundle_service.checkin(self.id, request)
@@ -438,13 +457,14 @@ class Worker:
         """
         Available disk space by bytes of this RunManager.
         """
-        error_msg = "Failed to run command {}".format("df " + self.work_dir)
+        error_msg = "Failed to run command {}".format("df -k" + self.work_dir)
         try:
-            p = Popen(["df", self.work_dir], stdout=PIPE)
+            # Option "-k" will ensure us with the returning disk space in 1KB units
+            p = Popen(["df", "-k", self.work_dir], stdout=PIPE)
             output, error = p.communicate()
             # Return None when there is an error.
             if error:
-                logger.error(error.strip())
+                logger.error(error.strip() + ": {}".format(error))
                 return None
 
             if output:
@@ -453,7 +473,7 @@ class Worker:
                 # The machine being attached as a worker may be using a different language other than
                 # English, so check the 4th header if "Available" is not present.
                 index = headers.index("Available") if "Available" in headers else 3
-                # We convert the original result from df command in unit of 1KB blocks into bytes.
+                # We convert the original result from df command in unit of 1KB units into bytes.
                 return int(lines[1].split()[index]) * 1024
 
         except Exception as e:
@@ -462,10 +482,20 @@ class Worker:
 
     def initialize_run(self, bundle, resources):
         """
-        First, checks in with the bundle service and sees if the bundle
-        is still assigned to this worker. If not, returns immediately.
+        First, checks if the worker has already finished receiving/starting the number of jobs allowed to run.
+        If not, returns immediately.
+        Then, checks in with the bundle service and sees if the bundle is still assigned to this worker.
+        If not, returns immediately.
         Otherwise, tell RunManager to create the run.
         """
+        if self.exit_after_num_runs == self.num_runs:
+            print(
+                'Worker has finished starting the number of jobs allowed to run on: {}. '
+                'Stop starting further runs.'.format(self.exit_after_num_runs),
+                file=sys.stdout,
+            )
+            return
+
         now = time.time()
         start_message = {'hostname': socket.gethostname(), 'start_time': int(now)}
 
@@ -507,6 +537,8 @@ class Worker:
                 finalized=False,
                 is_restaged=False,
             )
+            # Increment the number of runs that have been successfully started on this worker
+            self.num_runs += 1
         else:
             print(
                 'Bundle {} no longer assigned to this worker'.format(bundle['uuid']),
